#pragma once

#include <stdint.h>

namespace Devices::Mcu
{
    template <uint32_t ClockFrequency>
    struct Adc 
    {
        enum class Reference
        {
            ARef = 0,
            AVcc = 1,
            Internal = 3
        };

        enum class Prescaler
        {
            Div2 = 1,
            Div4 = 2,
            Div8 = 3,
            Div16 = 4,
            Div32 = 5,
            Div64 = 6,
            Div128 = 7,
        };

        enum class Trigger
        {
            FreeRunning = 0,
            AnalogComparator = 1,
            Int0 = 2,
            Timer0CompA = 3,
            Timer0Overflow = 4,
            Timer1CompB = 5,
            Timer1Overflow = 6,
            Timer1CaptureEvent = 7
        };

        static constexpr Prescaler CalculatePrescaler()
        {
            uint16_t minDivider = ClockFrequency / 200000;
            
            if (minDivider > 64)
            {
                return Prescaler::Div128;
            }
            else if (minDivider > 32)
            {
                return Prescaler::Div64;
            }
            else if (minDivider > 16)
            {
                return Prescaler::Div32;
            }
            else if (minDivider > 8)
            {
                return Prescaler::Div16;
            }
            else if (minDivider > 4)
            {
                return Prescaler::Div8;
            }
            else if (minDivider > 2)
            {
                return Prescaler::Div4;
            }

            return Prescaler::Div2;
        }


        static void Configure(Reference const reference, uint8_t channel, bool adjustLeft = false)
        {
            ADMUX = (((uint8_t) reference) << REFS0) | channel;

            if (adjustLeft)
            {
                ADMUX |= _BV(ADLAR);
            }

            if (channel < 6)
            {
                DIDR0 |= _BV(channel);
            }
        }


        static void EnableAutoTrigger(Trigger const trigger, bool enableInterrupt)
        {
            ADCSRB = (uint8_t) trigger;
            ADCSRA = _BV(ADIF);

            if (enableInterrupt)
            {
                ADCSRA |= _BV(ADIE);
            }

            ADCSRA |= _BV(ADEN) | _BV(ADATE) | (uint8_t) CalculatePrescaler();
        }
    };
}