#pragma once

#include <stdint.h>
#include "range.hpp"
#include "voltage.hpp"


namespace Voltmeter
{
    struct Measurement
    {
        Voltmeter::Range range;

        uint16_t samplesN;
        uint16_t min;
        uint16_t max;
        uint32_t sum;
        uint64_t sumOfSquares;

        Measurement()
            : range(Voltmeter::Range::_4V)
            , samplesN(0)
            , min(0xffff)
            , max(0)
            , sum(0)
            , sumOfSquares(0)
        {}

        Measurement(Voltmeter::Range const range, uint16_t const samplesN)
            : range(range)
            , samplesN(samplesN)
            , min(0xffff)
            , max(0)
            , sum(0)
            , sumOfSquares(0)
        {}

        Voltage GetMaxVoltage(uint8_t digitsN, uint32_t calibration);
        Voltage GetDcVoltage(uint8_t digitsN, uint32_t calibration);
        Voltage GetAcVoltage(uint8_t digitsN);
        uint32_t GetCalibration();
    };
}